module ELTest__assets

  class TagHelpers < E

    def js
      (src = params[:src]) ?
        js_tag(src: src)   :
        js_tag(params[:asset])
    end

    def css
      return css_tag(href: params[:href]) if params[:href]
      return css_tag(src:  params[:src])  if params[:src]
      css_tag(params[:asset])
    end

    def png
      (src = params[:src]) ?
      png_tag(src: src)    :
      png_tag(params[:asset])
    end
  end

  Spec.new self do
    app E.new {
      assets_url '/assets'
      mount TagHelpers
    }
    map TagHelpers.base_url

    Testing :js_tag do
      get :js, asset: :master
      does(/src="\/assets\/master\.js"/).match_body?

      Should 'ignore assets_url when :src given' do
        get :js, src: 'master'
        does(/src="master\.js"/).match_body?
      end
    end

    Testing :css_tag do
      get :css, asset: :master
      does(/href="\/assets\/master\.css"/).match_body?

      Should 'ignore assets_url when :href given' do
        get :css, href: 'http://some.cdn/master'
        does(/href="http:\/\/some.cdn\/master\.css"/).match_body?
      end

      Should 'also work with :src instead of :href' do
        get :css, src: 'foo'
        does(/href="foo\.css"/).match_body?
        refute(last_response.body) =~ /src=/
      end
    end

    Testing :png_tag do
      get :png, asset: :master
      does(/src="\/assets\/master\.png"/).match_body?

      Should 'ignore assets_url when :src given' do
        get :png, src: 'blah/master'
        does(/src="blah\/master\.png"/).match_body?
      end
    end

  end

  class AssetsMapper < E

    def get_assets_mapper type
      html = ''
      mapper = assets_mapper params[:baseurl]
      if src = params[:src]
        html << mapper.send(type, src: src)
      elsif href = params[:href]
        html << mapper.send(type, href: href)
      else
        html << mapper.send(type, params[:url])
      end
      html
    end

    def assets_mapper_chdir type
      html = ''
      mapper = assets_mapper params[:baseurl]
      params[:scenario].each do |scenario|
        path, file = scenario.split
        if file.nil?
          file = path
          path = nil
        end
        mapper.cd path
        html << mapper.send(type, file)
      end
      html
    end

    def assets_mapper_with_block type
      html, url = '', params[:url]
      assets_mapper params[:baseurl] do
        html << send(type, url)
      end
      html
    end

    def assets_mapper_with_suffix type, suffix
      html, url = '', params[:url]
      assets_mapper params[:baseurl], suffix: suffix do
        html << send(type, url)
      end
      html
    end
  end

  Spec.new AssetsMapper do

    Testing :js do
      get :assets_mapper, :js_tag, baseurl: './', url: 'master'
      does('src="./master.js"').match_body?
      
      get :assets_mapper, :js_tag, baseurl: '/', url: 'master'
      does('src="/master.js"').match_body?

      get :assets_mapper, :js_tag, baseurl: 'http://some.cdn', url: 'master'
      does('src="http://some.cdn/master.js"').match_body?

      Should 'skip baseurl' do
        get :assets_mapper, :js_tag, src: 'master', baseurl: 'skipit'
        does('src="master.js"').match_body?
      end
    end

    Testing :css do
      get :assets_mapper, :css_tag, baseurl: './', url: 'master'
      does('href="./master.css"').match_body?
      
      get :assets_mapper, :css_tag, baseurl: '/', url: 'master'
      does('href="/master.css"').match_body?

      get :assets_mapper, :css_tag, baseurl: 'http://some.cdn', url: 'master'
      does('href="http://some.cdn/master.css"').match_body?

      Should 'skip baseurl' do
        get :assets_mapper, :css_tag, href: 'master', baseurl: 'skipit'
        does('href="master.css"').match_body?
      end
    end

    Testing :png do
      get :assets_mapper, :png_tag, baseurl: './', url: 'master'
      does('src="./master.png"').match_body?
      
      get :assets_mapper, :png_tag, baseurl: '/', url: 'master'
      does('src="/master.png"').match_body?

      get :assets_mapper, :png_tag, baseurl: 'http://some.cdn', url: 'master'
      does('src="http://some.cdn/master.png"').match_body?

      Should 'skip baseurl' do
        get :assets_mapper, :png_tag, src: 'master', baseurl: 'skipit'
        does('src="master.png"').match_body?
      end
    end

    Testing :AssetsLoaderWithBlock do
      get :assets_mapper_with_block, :js_tag, url: 'master'
      does('src="master.js"').match_body?
      
      get :assets_mapper_with_block, :css_tag, url: 'master', baseurl: '/'
      does('href="/master.css"').match_body?

      get :assets_mapper_with_block, :css_tag, url: 'master', baseurl: './'
      does('href="./master.css"').match_body?

      get :assets_mapper_with_block, :png_tag, url:  'master', baseurl:  'http://some.cdn'
      does('src="http://some.cdn/master.png"').match_body?
    end

    Testing :AssetsLoaderWithSuffix do
      get :assets_mapper_with_suffix, :js_tag, '-sfx', url:  'master'
      does('src="master.js-sfx"').match_body?
      
      get :assets_mapper_with_suffix, :css_tag, '-sfx', url:  'master', baseurl:  '/'
      does('href="/master.css-sfx"').match_body?

      get :assets_mapper_with_suffix, :png_tag, '-sfx', url:  'master', baseurl:  'http://some.cdn'
      does('src="http://some.cdn/master.png-sfx"').match_body?
    end

    Testing :AssetsLoaderChdir do
      Should 'avoid redundant path traversal' do
        get :assets_mapper_chdir, :js_tag, scenario:  [ '../../etc/passwd jquery' ]
        does('src="etc/passwd/jquery.js"').match_body?

        get :assets_mapper_chdir, :js_tag, scenario:  [ '../etc/passwd jquery' ]
        does('src="etc/passwd/jquery.js"').match_body?

        get :assets_mapper_chdir, :js_tag, scenario:  ['vendor/jquery jquery', '../../../../ master']
        does('src="master.js"').match_body?
      end

      Should 'cd to vendor/jquery and load jquery.js' do
        get :assets_mapper_chdir, :js_tag, baseurl:  '/assets', scenario:  ['vendor/jquery jquery', '.. master', '/ master']
        does('src="/assets/vendor/jquery/jquery.js"').match_body?
        
        Then 'cd to .. and load vendor/master.js' do
          does('src="/assets/vendor/master.js"').match_body?
        end
        
        Then 'cd to / and load master.js' do
          does('src="/assets/master.js"').match_body?
        end
      end

      Should 'cd to vendor/jquery and load vendor/jquery/jquery.js' do
        get :assets_mapper_chdir, :js_tag, baseurl:  '/assets', scenario:  ['vendor/jquery jquery', '../.. master', '/scripts master']
        does('src="/assets/vendor/jquery/jquery.js"').match_body?
        
        Then 'cd to ../.. and load master.js' do
          does('src="/assets/master.js"').match_body?
        end
        
        Then 'cd to /scripts and load master.js' do
          does('src="/assets/scripts/master.js"').match_body?
        end
      end

      Should 'cd to css/themes and load css/themes/black.css' do
        
        get :assets_mapper_chdir, :css_tag, baseurl:  '/assets', scenario:  ['css/themes black', ' master', '/css master']
        does('href="/assets/css/themes/black.css"').match_body?
        
        Then 'cd to root and load master.css' do
          does('href="/assets/master.css"').match_body?
        end
        
        Then 'cd to /css and load master.css' do
          does('href="/assets/css/master.css"').match_body?
        end
      end

      Should 'behave well with rooted baseurls' do
        Should 'cd to vendor/icons/16x16 and load vendor/icons/16x16/file.png' do

          get :assets_mapper_chdir, :png_tag, baseurl:  '/public', scenario:  ['vendor/icons/16x16 file', '../.. sprite', '/icons folder']
          does('src="/public/vendor/icons/16x16/file.png"').match_body?

          Then 'cd to ../.. and load vendor/sprite.png' do
            does('src="/public/vendor/sprite.png"').match_body?
          end

          Then 'cd to /icons and load folder.png' do
            does('src="/public/icons/folder.png"').match_body?
          end
        end
      end

      Should 'behave well with protocoled baseurls' do
        Should 'cd to icons/16x16 and load icons/16x16/file.png' do
          
          get :assets_mapper_chdir, :png_tag, baseurl:  'http://some.cdn', scenario:  ['icons/16x16 file', '.. sprite', '/imgs img']
          does('src="http://some.cdn/icons/16x16/file.png"').match_body?
          
          Then 'cd to .. and load sprite.png' do
            does('src="http://some.cdn/icons/sprite.png"').match_body?
          end
          
          Then 'cd to /imgs and load img.png' do
            does('src="http://some.cdn/imgs/img.png"').match_body?
          end
        end
      end
    end
  end
end
