module ELTest__tag_factory
  Spec.new self do
    include EL::TagFactory

    Testing :doctype do
      check(doctype_tag) == "<!DOCTYPE html>\n"
      check(doctype_tag { :smth } ) == "<!DOCTYPE html>\nsmth"
      check(doctype_tag { p_tag :smth} ) == "<!DOCTYPE html>\n<p>smth</p>"
    end

    Should 'return closed empty tags when no text nor proc given' do
      expect(html_tag) == '<html></html>'
      expect(a_tag)    == '<a></a>'
      expect(div_tag)  == '<div></div>'
    end

    Should 'use first argument as content' do
      expect(h1_tag :foo) == '<h1>foo</h1>'
      expect(p_tag  :bar) == '<p>bar</p>'
    end

    Should 'use second arg as attrs' do
      expect(a_tag :foo, href: :bar) == '<a href="bar">foo</a>'
      expect(option_tag :label, value: :value) == '<option value="value">label</option>'
      Ensure 'multiple attrs properly converted to string' do
        expect(p_tag :smth, class: :foo, style: :bar) == '<p class="foo" style="bar">smth</p>'
      end
    end

    Should 'work when only attrs given' do
      expect(a_tag href: :foo, title: :bar) == '<a href="foo" title="bar"></a>'
    end

    Should 'work when attrs and proc given' do
      expect(html_tag(manifest: :foo) { :body }) == '<html manifest="foo">body</html>'
    end

    Should 'work well with nested procs' do
      expect(html_tag { div_tag :body }) == '<html>&lt;div&gt;body&lt;/div&gt;</html>'
      expect(html_tag! { div_tag! { a_tag(:foo) }}) == '<html><div><a>foo</a></div></html>'
      html = doctype_tag do
        html_tag! do
          head_tag! do
            meta_tag(charset: 'UTF-8') +
            title_tag(:smth)
          end +
          body_tag! do
            div_tag :smth
          end
        end
      end
      expect(html) == '<!DOCTYPE html>
<html><head><meta charset="UTF-8"><title>smth</title></head><body><div>smth</div></body></html>'
    end

    Testing 'empty tags' do
      expect(br_tag) == '<br>'
      expect(hr_tag) == '<hr>'
      expect(meta_tag charset: 'UTF-8') == '<meta charset="UTF-8">'
    end

    Testing :comment_tag do
      expect(comment_tag :smth)     == '<!-- smth -->'
      expect(comment_tag { :smth }) == '<!-- smth -->'
    end

    Testing :js_tag do
      does(js_tag 'url').match? 'src="url.js"'
      does(js_tag 'url').match? 'type="text/javascript"'

      Ensure 'url has priority over :src option' do
        does(js_tag 'foo', src: 'bar').match? 'src="foo.js"'
      end

      Should 'append suffix' do
        does(js_tag 'url', suffix: '-blah').match? 'src="url.js-blah"'
        does(js_tag 'foo', src: 'bar', suffix: '-blah').match? 'src="foo.js-blah"'
      end
    end

    Testing :css_tag do
      does(css_tag 'url').match? 'rel="stylesheet"'
      does(css_tag 'url').match? 'href="url.css"'

      Ensure 'url has priority over :src option' do
        does(css_tag 'foo', src: 'bar').match? 'href="foo.css"'
      end

      Should 'append suffix' do
        does(css_tag 'url', suffix: '-blah').match? 'href="url.css-blah"'
        does(css_tag 'foo', src: 'bar', suffix: '-blah').match? 'href="foo.css-blah"'
      end
    end

    Testing :png_tag do
      does(png_tag 'image').match? '<img src="image.png">'

      Ensure 'url has priority over :src option' do
        does(jpg_tag 'foo', src: 'bar').match? 'src="foo.jpg"'
      end

      Should 'append suffix' do
        does(gif_tag 'url', suffix: '-blah').match? 'src="url.gif-blah"'
        does(svg_tag 'foo', src: 'bar', suffix: '-blah').match? 'src="foo.svg-blah"'
      end
    end

  end

  class App < E
    map '/'
    format_for :foo, '.bar'
    format '.html'

    def index
    end

    def foo
    end

    def post_bar
    end
  end
  App.mount

  class App2 < E
    map 'app2'
    format '.bar'

    def foo
    end
  end
  App2.mount
  
  Spec.new self do
    app = App.new

    Testing :link_to do
      expect( app.link_to(:index) ) =~ %r[href="/index"]
      expect( app.link_to('index.html') ) =~ %r[href="/index.html"]

      expect( app.link_to(:foo) ) =~ %r[href="/foo">/foo]
      expect( app.link_to(:foo, 'some-label') ) =~ %r[href="/foo">some-label]
      
      expect( app.link_to(:foo, target: '_blank') ) =~
        %r[href="/foo" target="_blank">/foo]
      
      expect( app.link_to(:foo, 'some-label', target: '_blank') ) =~
        %r[href="/foo" target="_blank">some-label]
      
      expect( app.link_to(:foo) { 'some-label' } ) =~
        %r[href="/foo">some-label]

      expect( app.link_to(:foo, target: '_blank') { 'some-label' } ) =~
        %r[href="/foo" target="_blank">some-label]

      false?( app.link_to('foo.html') ) =~ %r[href="/foo.html"]
      expect( app.link_to('foo.bar') ) =~ %r[href="/foo.bar"]

      expect( app.link_to App2[:foo] ) =~ %r[href="/app2/foo"]
      expect( app.link_to App2['foo.bar'] ) =~ %r[href="/app2/foo.bar"]

      expect( app.link_to(:bar) ) =~ %r[href="/bar"]
      expect( app.link_to('bar.html') ) =~ %r[href="/bar.html"]
      expect( app.link_to(:baz) ) =~ %r[href="baz"]
      expect( app.link_to('baz.html') ) =~ %r[href="baz.html"]
    end

  end
end
